function [P2,P3,U]=lp_knownrotation(u1,u2,u3,R2,R3,d1,d2,d3);
% Feasibility test. Computes 2D points and cameras centres given known rotations
% for 3 views PROVIDED there exists feasible solution within angle errors
% d1,d2,d3 for each view, respectively.
% Assumption: P1=[I 0]
% INPUT:
%  u1,u2,u3 homogeneous 1d image coordinates
%  R2,R3: Rotation matrices for camera 2 and 3
%  d1,d2,d3: maximum allowed angular errors for view 1,2,3

R1=eye(2);
n=size(u1,2);

%PARAMETER ORDER: t2x,t2y,t3x,t3y,U1x,U1y,U2x,U2y,...
nbrparams=4 + 2*n;
A=zeros(0,nbrparams);

for ii=1:n
    %first camera constraints
    %|u x PU |<= tan(d1)*u'*PU
    A(end+1,4+2*ii+[-1,0])=  u1(1,ii)*R1(2,:)-u1(2,ii)*R1(1,:) - tan(d1)*u1(:,ii)'*R1;
    A(end+1,4+2*ii+[-1,0])=-(u1(1,ii)*R1(2,:)-u1(2,ii)*R1(1,:))- tan(d1)*u1(:,ii)'*R1;
    %second camera constraints
    %|u x PU |<= tan(d2)*u'*PU
    A(end+1,4+2*ii+[-1,0])=  u2(1,ii)*R2(2,:)-u2(2,ii)*R2(1,:) - tan(d2)*u2(:,ii)'*R2;
    A(end,1:2)= [-u2(2,ii),u2(1,ii)]-tan(d2)*u2(:,ii)';
    A(end+1,4+2*ii+[-1,0])=-(u2(1,ii)*R2(2,:)-u2(2,ii)*R2(1,:))- tan(d2)*u2(:,ii)'*R2;
    A(end,1:2)=-[-u2(2,ii),u2(1,ii)]-tan(d2)*u2(:,ii)';

    %third camera constraints
    %|u x PU |<= tan(d3)*u'*PU
    A(end+1,4+2*ii+[-1,0])=  u3(1,ii)*R3(2,:)-u3(2,ii)*R3(1,:) - tan(d3)*u3(:,ii)'*R3;
    A(end,3:4)= [-u3(2,ii),u3(1,ii)]-tan(d3)*u3(:,ii)';
    A(end+1,4+2*ii+[-1,0])=-(u3(1,ii)*R3(2,:)-u3(2,ii)*R3(1,:))- tan(d3)*u3(:,ii)'*R3;
    A(end,3:4)=-[-u3(2,ii),u3(1,ii)]-tan(d3)*u3(:,ii)';
end

%set scale by setting one element in U to one
[mm,indexpoints]=max(u1(1,:)); %this ensure positive depth for that point (in camera 1)
index=2*indexpoints+3;

b=-A(:,index);
A(:,index)=[];
f=zeros(nbrparams-1,1);
[x,fval,exitflag]=linprog(f,A,b);

if exitflag==1,
    %feasible solution
    P2=[R2,x(1:2)];
    P3=[R3,x(3:4)];
    %recover U from x (and insert "1" which was fixed)
    tmp=x(5:end);
    U=reshape([tmp(1:2*indexpoints-2);1;tmp(2*indexpoints-1:end)],2,n);
    U(3,:)=1;
else
    %no solution
    P2=[];P3=[];U=[];
end
if 0,
    tmp=U;tmp(3,:)=[];
    x=[P2(:,3);P3(:,3);tmp(:)];
end

