function [U,P]=linf_referenceplane(u,A,iter,upper);
%L-infinity structure and motion with a reference plane
%[U,P] = LINF_REFERENCEPLANE(u,A,iter,upper);
%
%u is a cell list of 2-by-m matrices of measured image coordinates.
% NB: Missing image points should be replaced by NaN.
%A is a cell list of 3-by-3 matrices of inter-homographies w.r.p. to the first camera
%iter (optional) is the number of bisection iterations. Default is 15.
%upper (optional) is the upper bound in pixels for initial interval. Default is 1.
%U is a 4-by-1 vector of 3D homogeneous point coordinates.
%P is a cell list of 3-by-4 camera matrices.
%
% NB: All points must be on one side of the reference plane
%     (This will be fixed for later versions).
% (C) 2005 Fredrik Kahl.

if nargin<3 | isempty(iter),
    iter = 15;
end

if nargin<4 | isempty(upper),
    upper = 1;
end

nbrimages=length(u);
if nbrimages<2,
    error('too few images');
end

if length(A)~=nbrimages-1,
    error('wrong number of homographies');
end


Atmp=cell(1,nbrimages);
Atmp{1}=eye(3);
Atmp(2:end)=A;

%Plane at infinity is used as ref plane here
[U,P]=linf_knownrotation(u,Atmp,iter,upper);
