%Test of algorithms with L-infinity norm
% Version 1.0

%PATHS:
%addpath c:\Program\MATLAB704\work\SeDuMi105\

disp('--------------------------');
disp('Demo of L-infinity toolbox');
disp('Version 1.0');
disp('(C) 2005 Fredrik Kahl');
disp('--------------------------');
disp('This file contains examples of how to call the algorithms.');
disp('Remember to set the path to SeDuMi first.');

disp('pause');pause

%number of views
nbrimages=5
nbrpoints=10

option={'noise=0.01','calibrated','cube=1','distance=2'};
[U,P,u,u0]=randomdata(nbrimages,nbrpoints,option);
disp('Some random points have been generated and');
disp('then corrupted with i.i.d image noise.');

disp('Root mean square of reprojection errors of synthetic solution:');
rms(U,P,u);
    
disp('pause');pause

%extract one point in each image for triangulation
utmp=zeros(2,nbrimages);
for ii=1:nbrimages,
    utmp(:,ii)=u{ii}(1:2,1);
end

disp('Algorithm: Triangulation');
Uinf=linf_triangulation(utmp,P);

disp('One triangulated point: L-infinity, synthetic');
[Uinf,U(:,1)]

disp('pause');pause

disp('Algorithm: Camera resectioning');

%extract points from first image
utmp=u{1};

Pinf=linf_resection(utmp,U);

disp('Cameras: L-infinity, synthetic');
Pinf/norm(Pinf)
P{1}/norm(P{1})

disp('pause');pause

option{end+1}='planar'; %generated on z=0-plane
[Up,Pplane,uplane,uplane0]=randomdata(nbrimages,nbrpoints,option);

disp('Some random PLANAR points generated and');
disp('then corrupted with i.i.d image noise');

Uplane=Up([1,2,4],:);
H=Pplane{1}(:,[1,2,4]);

disp('Algorithm: Homography estimation');
Hinf=linf_homography(uplane{1},Uplane);

disp('Homographies: L-infinity, synthetic');
Hinf/norm(Hinf)
H/norm(H)

disp('pause');pause

disp('Algorithm: Structure and motion with known camera rotation');

A=cell(1,nbrimages);
for ii=1:nbrimages,
    A{ii}=P{ii}(:,1:3);
end

[Uinf,Pinf]=linf_knownrotation(u,A);

disp('Reprojection errors: L-infinity solution');
rms(Uinf,Pinf,u);

disp('Reprojection errors: Synthetic');
rms(U,P,u);

disp('pause');pause

disp('Algorithm: Structure and motion with a reference plane');

disp('Assumption: Inter-image homographies are known.');
H=cell(1,nbrimages-1);
H1=P{1}(:,[1,2,4]);
for ii=1:nbrimages-1,
    Hii=P{ii+1}(:,[1,2,4]);
    H{ii}=Hii*inv(H1);
end

[Uinf,Pinf]=linf_referenceplane(u,H);

disp('Reprojection errors: L-infinity solution');
rms(Uinf,Pinf,u);

disp('Reprojection errors: Synthetic');
rms(U,P,u);

disp('The end');

