function [d,ok,singularvalues]=depthfromxandy(x,y);
% [d,ok,singularvalues]=depthfromxandy(x,y) - calculates depth d
% given 3D object points in homogeneous coordinates x 
% and 2D image points in homogeneous coordinates y 

x=pflat(x);
y=pflat(y);
[temp,n]=size(x);
x=rnorm(x);
y=rnorm(y);
m=[ ...
-x(1,5:n)' zeros(n-4,1) zeros(n-4,1) -y(1,4)*x(4,5:n)' diag(y(1,5:n)); ...
zeros(n-4,1) -x(2,5:n)' zeros(n-4,1) -y(2,4)*x(4,5:n)' diag(y(2,5:n)); ...
zeros(n-4,1) zeros(n-4,1) -x(3,5:n)' -y(3,4)*x(4,5:n)' diag(y(3,5:n))];
   % the depth vector should fulfill m*d' = zeros.
   % One way to solve it is to use svd directly on m.
% Oldest version.
% [u1,ss1,v1]=svd(m);
% d2 = v1(:,n);
   % Another way is to solve m'*m*d'=s*d'=0.
   % Observe that s has a diagonal submatrix i.e. s(5:n,5:n) is
   % diagonal with strictly positive elements. So the expensive
   % matrix calculations can be simplified.
% Old version.
%s=m'*m;
%t=s(1:4,1:4)-s(1:4,5:n)*inv(s(5:n,5:n))*s(5:n,1:4);
%[u,ss,v]=svd(t);
%dtop=v(:,4);
%dbot=-inv(s(5:n,5:n))*s(5:n,1:4)*dtop;
%dold=[dtop;dbot]'

% Using the diagonal structure of s(5:n,5:n) d can be calculated as.
% observe that sdiag = diag(s(5:n,5:n))'
sdiag=sum(y(:,5:n).^2);
invsdiag=ones(1,n-4)./sdiag;
mtop=m(:,1:4);
mbottom = m(:,5:n);
A=mtop'*mtop;
B=mtop'*mbottom;
t=A-B*((invsdiag'*ones(1,4)).*B');
[u,ss,v]=svd(t);
singularvalues=diag(ss);
ok=singularvalues(4)<0.0001;
dtop=v(:,4);
dbot=-((invsdiag'*ones(1,4)).*B')*dtop;
d=[dtop;dbot]';

% Fix so that all elements are positive if possible.
if sum(d)<0,
 d=-d/norm(d);
else
 d=d/norm(d);
end;
