function out_image = scale(in_image, scale_factor, interpolation, cubic_factor);

% SCALE Scales a matrix, interpreted as an image. 
%
%	SCALE(image, scale_factor) returns an image, scaled by a
%	factor 'scale_factor', using pixel replication. 'scale_factor'
%	can either be a scalar or an 2 by 1 matrix, where the first
%	element gives the x-scale, and the second the y-scale.
%
%	SCALE(image, scale_factor, interpolation), is as above, but 
%	where interpolation can be one of:
%	'p' (pixel replication - the default), 'b' (bilinear) 
%	or 'c' (cubic spline).
%
%	SCALE(image, scale_factor, 'c', cubic_factor) scales the image
%	using the cubic spline method, where 'cubic_factor' should
%	preferably be -1..-0.5 for appealing results. The default
%	is -1.

if nargin < 2, error('Too few input arguments.'); end
if nargin == 2, interpolation = 'p'; cubic_factor = ' ';end
if nargin == 3,
  if interpolation ~= 'p' & interpolation ~= 'b' & interpolation ~= 'c',
    error('Unknown choise of interpolation method.');
  end
  if interpolation =='c',
    cubic_factor = -1;
  else
    cubic_factor = ' ';
  end
end
if nargin == 4,
  if interpolation ~= 'p' & interpolation ~= 'b' & interpolation ~= 'c',
    error('Unknown choise of interpolation method.');
  end
  if interpolation ~='c', 
    error('To many arguments for the choosed interpolation method.');
  end
end

if interpolation =='c',
  size_cubic_factor=prod(size(cubic_factor));
  if size_cubic_factor == 0,
    error('Empty cubic-spline factor.');
  elseif size_cubic_factor > 1,
    error('Cubic-spline factor must be scalar.');
  end
end

size_scale=prod(size(scale_factor));
if size_scale == 0,
  error('No scale given.');
elseif size_scale == 1,
  scale_factor(2) = scale_factor(1);
elseif size_scale > 2,
  error('Can only scale in 2 dimensions.');
end

if scale_factor(1) <= 0 | scale_factor(2) <= 0,
  error('Scale factor must be positive.');
end

temp_file_name1 = ['temp_file', num2str(rand)];
temp_file_name2 = ['temp_file', num2str(rand)];

savepgm(in_image, temp_file_name1);
eval(['!/usr/matematik/vision/bin/pgmscale -x ', num2str(scale_factor(1)), ' -y ', num2str(scale_factor(2)), ' -', interpolation,' ', num2str(cubic_factor), ' ', temp_file_name1, ' > ', temp_file_name2 ]);
out_image = readpgm(temp_file_name2);
eval('!/usr/bin/rm -f temp_file?.*');

